@@ -5,22 +5,23 @@ This module is uesd for statistical analysis
 '''
 import numpy as np
 from matplotlib import pyplot as plt
-from scipy.integrate import quad
+from maysics.calculus import inte
+from maysics.utils import grid_net
 from scipy.stats import chi2
-from scipy.optimize import minimize
 from scipy.interpolate import interp1d
 
 
-def r_moment(data, a=None, b=None, k=1):
+def r_moment(data, p_range=None, args={}, k=1, acc=0.1):
     '''
     计算原点矩
     
     参数
     ----
     data：一维列表或函数类型，样本或概率密度函数
-    a：浮点数类型，可选，计算下限
-    b：浮点数类型，可选，计算上限
-    k：整型，可选，阶数
+    p_range：元组类型，可选，计算上下限
+    args：字典类型，可选，用于传递func中的其他参数
+    k：整型，可选，阶数，默认为1
+    acc：浮点数类型，可选，积分精度，仅在data为函数类型时有效，默认为0.1
     
     返回
     ----
@@ -32,34 +33,36 @@ def r_moment(data, a=None, b=None, k=1):
     Parameters
     ----------
     fdata: 1-D list or function, a sample or probability density function
-    a: float, callable, lower limit of calculation
-    b: float, callable, upper limit of calculation
-    k: int, callable, orders
+    p_range：tuple, callable, lower and upper limit of calculation
+    args: dict, callable, pass other parameters to func
+    k: int, callable, orders, default=1
+    acc: float, callable, integration accuracy, it's effective only when data is function, default=0.1
     
     Return
     ------
     num, raw moment
     '''
-    if type(data).__name__ == 'function':
+    if type(data).__name__ == 'function' or type(data).__name__ == 'method':
         def ex_func(x):
-            return x**k * data(x)
+            return x**k * data(x, **args)
         
-        return quad(ex_func, a, b)[0]
+        return inte(ex_func, [p_range], acc=acc)
     
     else:
         data = np.array(data)**k
         return data.mean()
 
 
-def ex(data, a=None, b=None):
+def ex(data, p_range=None, args={}, acc=0.1):
     '''
     计算数学期望，等价于一阶原点矩
     
     参数
     ----
     data：一维列表或函数类型，样本或概率密度函数
-    a：浮点数类型，可选，计算下限
-    b：浮点数类型，可选，计算上限
+    p_range：元组类型，可选，计算上下限
+    args：字典类型，可选，用于传递func中的其他参数
+    acc：浮点数类型，可选，积分精度，仅在data为函数类型时有效，默认为0.1
     
     返回
     ----
@@ -71,34 +74,36 @@ def ex(data, a=None, b=None):
     Parameters
     ----------
     fdata: 1-D list or function, a sample or probability density function
-    a: float, callable, lower limit of calculation
-    b: float, callable, upper limit of calculation
+    p_range：tuple, callable, lower and upper limit of calculation
+    args: dict, callable, pass other parameters to func
+    acc: float, callable, integration accuracy, it's effective only when data is function, default=0.1
     
     Return
     ------
     num, mathematical expectation
     '''
-    if type(data).__name__ == 'function':
+    if type(data).__name__ == 'function' or type(data).__name__ == 'method':
         def ex_func(x):
-            return x * data(x)
+            return x * data(x, **args)
         
-        return quad(ex_func, a, b)[0]
+        return inte(ex_func, [p_range], acc=acc)
     
     else:
         data = np.array(data)
         return data.mean()
 
 
-def c_moment(data, k, a=None, b=None):
+def c_moment(data, p_range=None, args={}, k=1, acc=0.1):
     '''
     计算中心矩
     
     参数
     ----
     data：一维列表或函数类型，样本或概率密度函数
-    a：浮点数类型，可选，计算下限
-    b：浮点数类型，可选，计算上限
-    k：整型，可选，阶数
+    p_range：元组类型，可选，计算上下限
+    args：字典类型，可选，用于传递func中的其他参数
+    k：整型，可选，阶数，默认为1
+    acc：浮点数类型，可选，积分精度，仅在data为函数类型时有效，默认为0.1
     
     返回
     ----
@@ -110,36 +115,38 @@ def c_moment(data, k, a=None, b=None):
     Parameters
     ----------
     data: 1-D list or function, a sample or probability density function
-    a: float, callable, lower limit of calculation
-    b: float, callable, upper limit of calculation
-    k: int, callable, orders
+    p_range：tuple, callable, lower and upper limit of calculation
+    args: dict, callable, pass other parameters to func
+    k: int, callable, orders, default=1
+    acc: float, callable, integration accuracy, it's effective only when data is function, default=0.1
     
     Return
     ------
     num, central moment
     '''
-    raw_exp = ex(data, a, b)
+    raw_exp = ex(data, p_range, args)
     
-    if type(data).__name__ == 'function':
+    if type(data).__name__ == 'function' or type(data).__name__ == 'method':
         def ex_func(x):
-            return (x-raw_exp)**k * data(x)
+            return (x-raw_exp)**k * data(x, **args)
         
-        return quad(ex_func, a, b)[0]
+        return inte(ex_func, [p_range], acc=acc)
     
     else:
         data = np.array(data) - raw_exp
         return r_moment(data, k=k)
 
 
-def dx(data, a=None, b=None):
+def dx(data, p_range=None, args={}, acc=0.1):
     '''
     计算方差，等价于二阶中心矩
     
     参数
     ----
     data：一维列表或函数类型，样本或概率密度函数
-    a：浮点数类型，可选，计算下限
-    b：浮点数类型，可选，计算上限
+    p_range：元组类型，可选，计算上下限
+    args：字典类型，可选，用于传递func中的其他参数
+    acc：浮点数类型，可选，积分精度，仅在data为函数类型时有效，默认为0.1
     
     返回
     ----
@@ -151,25 +158,27 @@ def dx(data, a=None, b=None):
     Parameters
     ----------
     data: 1-D list or function, a sample or probability density function
-    a: float, callable, lower limit of calculation
-    b: float, callable, upper limit of calculation
+    p_range：tuple, callable, lower and upper limit of calculation
+    args: dict, callable, pass other parameters to func
+    acc: float, callable, integration accuracy, it's effective only when data is function, default=0.1
     
     Return
     ------
     num, variance
     '''
-    return c_moment(data, 2, a, b)
+    return c_moment(data, p_range, args, 2, acc)
 
 
-def skew(data, a=None, b=None):
+def skew(data, p_range=None, args={}, acc=0.1):
     '''
     计算偏度，等价于三阶中心矩
     
     参数
     ----
     data：一维列表或函数类型，样本或概率密度函数
-    a：浮点数类型，可选，计算下限
-    b：浮点数类型，可选，计算上限
+    p_range：元组类型，可选，计算上下限
+    args：字典类型，可选，用于传递func中的其他参数
+    acc：浮点数类型，可选，积分精度，仅在data为函数类型时有效，默认为0.1
     
     返回
     ----
@@ -181,25 +190,27 @@ def skew(data, a=None, b=None):
     Parameters
     ----------
     data: 1-D list or function, a sample or probability density function
-    a: float, callable, lower limit of calculation
-    b: float, callable, upper limit of calculation
+    p_range：tuple, callable, lower and upper limit of calculation
+    args: dict, callable, pass other parameters to func
+    acc: float, callable, integration accuracy, it's effective only when data is function, default=0.1
     
     Return
     ------
     num, skewness
     '''
-    return c_moment(data, 3, a, b)
+    return c_moment(data, p_range, args, 3, acc)
 
 
-def kurt(data, a=None, b=None):
+def kurt(data, p_range=None, args={}, acc=0.1):
     '''
     计算峰度，等价于四阶中心矩
     
     参数
     ----
     data：一维列表或函数类型，样本或概率密度函数
-    a：浮点数类型，可选，计算下限
-    b：浮点数类型，可选，计算上限
+    p_range：元组类型，可选，计算上下限
+    args：字典类型，可选，用于传递func中的其他参数
+    acc：浮点数类型，可选，积分精度，仅在data为函数类型时有效，默认为0.1
     
     返回
     ----
@@ -211,17 +222,18 @@ def kurt(data, a=None, b=None):
     Parameters
     ----------
     data: 1-D list or function, a sample or probability density function
-    a: float, callable, lower limit of calculation
-    b: float, callable, upper limit of calculation
+    p_range：tuple, callable, lower and upper limit of calculation
+    args: dict, callable, pass other parameters to func
+    acc: float, callable, integration accuracy, it's effective only when data is function, default=0.1
     
     Return
     ------
     num, kurtosis
     '''
-    return c_moment(data, 4, a, b)
+    return c_moment(data, p_range, args, 4, acc)
 
 
-def mle(func, data, num, p_range=(-1, 1), method=None, tol=None):
+def mle(func, data, p_range, acc=0.1):
     '''
     最大似然法
     求解似然函数L(θ) = ln(∏ f(xi; θ)) = ∑ ln(f(xi; θ))在θ ∈ p_range范围内的最大值
@@ -231,11 +243,9 @@ def mle(func, data, num, p_range=(-1, 1), method=None, tol=None):
     func：函数类型，密度概率函数，func的形式应如下：
         def func(x, param):
         其中x需输入一个数或一个向量，是随机变量；param是一维ndarray，为未确定的参数
-    data：一维或二维列表，样本
-    num：未确定参数个数
-    p_range：一维或二维列表，未确定参数的取值范围，若为一维列表，则代表所有未确定参数的取值范围都一样，默认为(-1, 1)
-    method：字符串类型，求最大值的方法，可选'Nelder-Mead'、'Powell'、'CG'、'BFGS'、'Newton-CG'、'L-BFGS-B'、'TNC'、'COBYLA'、'SLSQP'、'trust-constr'、'dogleg'、'trust-ncg'、'trust-exact'、'trust-krylov'
-    tol：误差
+    data：一维或二维数组，样本
+    p_range：二维数组，未确定参数的取值范围
+    acc：数类型，可选，精度，默认为0.1
     
     返回
     ----
@@ -250,37 +260,38 @@ def mle(func, data, num, p_range=(-1, 1), method=None, tol=None):
     func：function, probability density function, the form of func should be as follows：
         def func(x, param):
         x is a number or a vector, random variable; param is 1-D ndarray, undetermined parameters
-    data：1-D or 2-D list，samples
-    num：the number of undetermined parameters
-    p_range：1-D or 2-D list, value range of undetermined parameters, in case of one-dimensional list，all the undetermined parameters are in the same range, default=(-1, 1)
-    method：str，the method of finding the maximum, 'Nelder-Mead'、'Powell'、'CG'、'BFGS'、'Newton-CG'、'L-BFGS-B'、'TNC'、'COBYLA'、'SLSQP'、'trust-constr'、'dogleg'、'trust-ncg'、'trust-exact'、'trust-krylov' are optional
-    tol：error
+    data：1-D or 2-D array，samples
+    p_range：2-D array, callable, value range of undetermined parameters
+    acc：num, callable, accuracy, default=0.1
     
     Return
     ------
     1-D ndarray, the best parameter value
     '''
     # 构建似然函数的相反数
-    def Lmin(theta):
+    def Lmin(theta, data):
         result = 0
         for i in data:
-            result += np.log(func(i, theta))
-        return result * (-1)
-
-    x0 = []
-    constraints = []
+            single_result = func(i, theta)
+            if single_result <= 0:
+                return float('-inf')
+            result += np.log(single_result)
+        return result
+    
     p_range = np.array(p_range)
-    if len(p_range.shape) == 1:
-        p_range = np.tile(p_range, (num, 1))
-    for i in range(num):
-        x0.append(p_range[i][0])
-        constraints.append({'type':'ineq', 'fun':lambda x: x[i] - p_range[i][0]})
-        constraints.append({'type':'ineq', 'fun':lambda x: p_range[i][1] - x[i]})
-    x0 = np.array(x0)
-        
-    res = minimize(Lmin, x0, method=method, constraints=constraints, tol=tol)
-        
-    return res.x
+    p_range_net = []
+    for i in p_range:
+        p_range_net.append(np.arange(i[0], i[1], acc))
+    
+    p_range_net = grid_net(*p_range_net)
+    
+    y = []
+    for i in p_range_net:
+        y.append(Lmin(i, data))
+    y = np.array(y)
+    x = np.argmax(y)
+    
+    return p_range_net[x]
 
 
 class DF1d():
@@ -291,7 +302,7 @@ class DF1d():
     参数
     ----
     sample：1-D列表，样本点
-    span：1-D列表，区间间隔，如span = [a, b, c]则将区间分为[a, b]和[b, c]，并统计各区间频率
+    span：1-D数组，区间间隔，如span = [a, b, c]则将区间分为[a, b]和[b, c]，并统计各区间频率
     kind：浮点数类型或整型，可选，将插值类型指定为字符串('linear'、'nearest'、'zero'、'slinear'、'squardic'、'previous'、'next'，其中'zero'、'slinear'、'squared'和'cubic'表示零阶、一阶、二阶或三阶样条曲线插值；'previous'和'next'只返回点的上一个或下一个值)或作为一个整数指定要使用的样条曲线插值器的顺序。默认为'linear'
     
     属性
@@ -305,7 +316,7 @@ class DF1d():
     Parameters
     ----------
     sample: 1-D list, samples
-    span: 1-D list, span of intervals, e.g. span = [a, b, c] will divide the interval into [a, b] and [b, c], and count the frequency of each interval
+    span: 1-D array, span of intervals, e.g. span = [a, b, c] will divide the interval into [a, b] and [b, c], and count the frequency of each interval
     kind: float or int, callable, specifies the kind of interpolation as a string ('linear', 'nearest', 'zero', 'slinear', 'quadratic', 'cubic', 'previous', 'next', where 'zero', 'slinear', 'quadratic' and 'cubic' refer to a spline interpolation of zeroth, first, second or third order; 'previous' and 'next' simply return the previous or next value of the point) or as an integer specifying the order of the spline interpolator to use. default='linear'
     
     Attribution
@@ -373,7 +384,7 @@ class DFT():
     
     参数
     ----
-    func_type：字符串类型，分布拟合采用的函数的类型，可选'pdf'(概率密度函数)、'cdf'(分布函数)和'dis'(离散分布函数)，默认为'pdf'
+    func_type：字符串类型，分布拟合采用的函数的类型，可选'pdf'(概率密度函数)、'cdf'(分布函数)和'pmf'(离散分布函数)，默认为'pdf'
     
     属性
     ----
@@ -386,7 +397,7 @@ class DFT():
 
     Parameter
     ---------
-    func_type: str, callable, types of functions used in distribution fitting, 'pdf'(probability density function), 'cdf'(cumulative distribution function) and 'dis'(discrete distribution function) are optional, default='pdf'
+    func_type: str, callable, types of functions used in distribution fitting, 'pdf'(probability density function), 'cdf'(cumulative distribution function) and 'pmf'(discrete distribution function) are optional, default='pdf'
     
     Atrributes
     ----------
@@ -394,11 +405,8 @@ class DFT():
     chi2_value: chi square value
     P: Probability of error in rejecting hypothesis
     '''
-    def __init__(self, func_type=None):
-        if not func_type:
-            self.__func_type = 'pdf'
-        else:
-            self.__func_type = func_type
+    def __init__(self, func_type='pdf'):
+        self.__func_type = func_type
     
     
     def __merge(self, data_list, pro_list):
@@ -434,14 +442,11 @@ class DFT():
                 pro_list[-2] += pro_list[-1]
                 data_list = np.delete(data_list, -1)
                 pro_list = np.delete(pro_list, -1)
-            
-        # 计算fi^2数组
-        data_list = data_list**2
         
         return data_list, pro_list
     
     
-    def __con_fit(self, data, func, num_data):
+    def __con_fit(self, data, func, num_data, args, acc):
         data = sorted(data)
         
         # 从小到大按20个一组分成多个部分
@@ -461,11 +466,11 @@ class DFT():
         pro_list = []
         if self.__func_type == 'pdf':
             for i in data_list:
-                pro_list.append(quad(func, i[0], i[-1])[0])
+                pro_list.append(inte(func, [[i[0], i[-1]]], args=args, acc=acc))
         
         elif self.__func_type == 'cdf':
             for i in data_list:
-                pro_list.append(func(i[-1]) - func(i[0]))
+                pro_list.append(func(i[-1], **args) - func(i[0], **args))
         
         pro_list = np.array(pro_list) * num_data
         
@@ -473,13 +478,11 @@ class DFT():
         for i in range(len(data_list)):
             data_list[i] = len(data_list[i])
         data_list = np.array(data_list)
-        
         data_list, pro_list = DFT.__merge(self, data_list, pro_list)
-
         return data_list, pro_list
     
 
-    def __dis_fit(self, data, func, num_data):
+    def __dis_fit(self, data, func, num_data, args):
         data = list(data)
         data_set = sorted(set(data))
 
@@ -488,7 +491,7 @@ class DFT():
         pro_list = []
         for i in data_set:
             data_list.append(data.count(i))
-            pro_list.append(func(i))
+            pro_list.append(func(i, **args))
         data_list = np.array(data_list)
         pro_list = np.array(pro_list) * num_data
 
@@ -497,38 +500,45 @@ class DFT():
         return data_list, pro_list
     
     
-    def fit(self, data, func):
+    def fit(self, data, func, args={}, acc=0.1):
         '''
         参数
         ----
         data：数据
         func：函数类型，分布函数或概率密度函数，函数的输入须为一个数
+        args：字典类型，可选，用于传递func中的其他参数
+        acc：浮点数类型，可选，积分精度，仅func_type为'pdf'或'cdf'时有效，默认为0.1
         
         
         Parameters
         ----------
         data: data
         func: function, cumulative distribution function or probability density function, the input of func should be a number
+        args: dict, callable, pass other parameters to func
+        acc: float, callable, integration accuracy, it's effective only when func_type is 'pdf' or 'cdf', default=0.1
         '''
         self.__data = np.array(data, dtype=np.float)
         self.__func = func
+        self.__args = args
         num_data = len(data)
         
-        if self.__func_type != 'dis':
-            data_list, pro_list = self.__con_fit(data, func, num_data)
+        if self.__func_type != 'pmf':
+            data_list, pro_list = self.__con_fit(data, func, num_data, args, acc)
         else:
-            data_list, pro_list = self.__dis_fit(data, func, num_data)
+            data_list, pro_list = self.__dis_fit(data, func, num_data, args)
         
         self.degree = len(pro_list) - 1
-        self.chi2_value = sum(data_list / pro_list) - num_data
-        self.P = 1 - chi2.cdf(self.chi2_value, self.degree)
+        self.chi2_value = sum((data_list - pro_list)**2 / pro_list)
+        self.P = chi2.cdf(self.chi2_value, self.degree)
     
     
     def __image_process(self, acc):
         x_min = min(self.__data)
         x_max = max(self.__data)
+        if self.__func_type == 'pmf':
+            acc = 1
         x = np.arange(x_min, x_max, acc)
-        y = self.__func(x)
+        y = self.__func(x, **self.__args)
         
         func_type = type(y).__name__
         if func_type == 'int' or func_type == 'float':
@@ -548,7 +558,12 @@ class DFT():
         plt.hlines(1, x_min, x_max, color='k', linestyle='-', lw=1)
         plt.vlines(self.__data.mean(), 0.9, 1.1, color='yellow', linestyle='-', lw=2)
         
-        plt.plot(x, y)
+        if self.__func_type != 'pmf':
+            plt.plot(x, y)
+        else:
+            plt.scatter(x, y)
+            zeros = np.zeros_like(y)
+            plt.vlines(x, zeros, y)
         plt.yticks([])
     
     
@@ -558,18 +573,19 @@ class DFT():
         
         参数
         ----
-        acc：浮点数类型，可选，画图精度，默认为0.01
+        acc：浮点数类型，可选，画图精度，仅func_type为'pdf'或'cdf'时有效，默认为0.01
         
         
         Display the violin chart of data and the image of hypothetical cumulative distribution function or probability density function
         
         Parameter
         ---------
-        acc: float, callable, the accuracy of drawing, default=0.01
+        acc: float, callable, the accuracy of drawing, it's effective only when method is 'pdf' or 'cdf', default=0.01
         '''
         self.__image_process(acc)
         plt.show()
     
+    
     def savefig(self,filename, acc=0.01):
         '''
         储存数据的小提琴图和假设的分布函数或概率密度图像
@@ -577,7 +593,7 @@ class DFT():
         参数
         ----
         filename：字符串类型，文件名
-        acc：浮点数类型，可选，画图精度，默认为0.01
+        acc：浮点数类型，可选，画图精度，仅在func_type为'pdf'或'cdf'时有效，默认为0.01
         
         
         Save the violin chart of data and the image of hypothetical cumulative distribution function or probability density function
@@ -585,7 +601,7 @@ class DFT():
         Parameters
         ----------
         filename: str, file name
-        acc: float, callable, the accuracy of drawing, default=0.01
+        acc: float, callable, the accuracy of drawing, it's effective only when method is 'pdf' or 'cdf', default=0.01
         '''
         self.__image_process(acc)
         plt.savefig(filename)
\ No newline at end of file
